import re
import requests_mock
from src.parallel_calls import ParallelCalls
import unittest

class TestParallelCalls(unittest.TestCase):
    
    @requests_mock.Mocker()
    def test_path_parallel_calls_ok(self, mocker):
        parallel_calls = ParallelCalls()
        ids = list(range(3))
        URL = "https://reqres.in/api/users"
        matcher = re.compile("https://reqres.in/api/users/*")
        mocker.register_uri('GET', matcher, json = {'a': 'b'})
        response = parallel_calls.path_calls(URL, ids)
        assert len(response) == 3
        assert response[0].get('status_code') != None
        assert response[0].get('result') != None

    @requests_mock.Mocker()
    def test_query_param_parallel_calls_ok(self, mocker):
        parallel_calls = ParallelCalls()
        ids = list(range(3))
        URL = "https://reqres.in/api/users?page="
        matcher = re.compile(r"https://reqres.in/api/users\?page=*")
        mocker.register_uri('GET', matcher, json = {'a': 'b'})
        response = parallel_calls.query_param_calls(URL, "page", ids)
        assert len(response) == 3
        assert response[0].get('status_code') != None
        assert response[0].get('result') != None

    @requests_mock.Mocker()
    def test_path_parallel_calls_non_200(self, mocker):
        parallel_calls = ParallelCalls()
        ids = list(range(3))
        URL = "https://reqres.in/api/users"
        matcher = re.compile("https://reqres.in/api/users/*")
        mocker.register_uri('GET', matcher, json = {'a': 'b'}, status_code = 403)
        response = parallel_calls.path_calls(URL, ids)
        assert len(response) == 3
        assert response[0].get('status_code') != 200
        assert response[0].get('result') == None
    
    @requests_mock.Mocker()
    def test_query_param_parallel_calls_non_200(self, mocker):
        parallel_calls = ParallelCalls()
        ids = list(range(3))
        URL = "https://reqres.in/api/users?page="
        matcher = re.compile(r"https://reqres.in/api/users\?page=*")
        mocker.register_uri('GET', matcher, json = {'a': 'b'}, status_code = 403)
        response = parallel_calls.query_param_calls(URL, "page", ids)
        assert len(response) == 3
        assert response[0].get('status_code') != 200
        assert response[0].get('result') == None

    @requests_mock.Mocker()
    def test_path_parallel_calls_exception(self, mocker):
        parallel_calls = ParallelCalls()
        ids = list(range(3))
        URL = "https://reqres.in/api/users"
        response = parallel_calls.path_calls(URL, ids)
        assert len(response) == 3
        assert response[0].get('status_code') == -1
        assert response[0].get('result') == None

    @requests_mock.Mocker()
    def test_query_param_parallel_calls_exception(self, mocker):
        parallel_calls = ParallelCalls()
        ids = list(range(3))
        URL = "https://reqres.in/api/users?page="
        response = parallel_calls.query_param_calls(URL, "page", ids)
        assert len(response) == 3
        assert response[0].get('status_code') == -1
        assert response[0].get('result') == None


import requests
from concurrent.futures import ThreadPoolExecutor, wait

class ParallelCalls:
    def __init__(self, number_of_threads=10):
        self.number_of_threads = number_of_threads

    def _get_single_value(self, URL, resource_id, query_param_name=None):
        ret_val = {}
        try:
            if query_param_name:
                URL = f"{URL}?{query_param_name}={resource_id}"
            else:
                URL = f"{URL}/{resource_id}"
            response = requests.get(URL)
            ret_val['status_code'] = response.status_code
            
            if response.status_code == 200:
                ret_val['result'] = response.json()
            else:
                ret_val['result'] = None
        except:
            ret_val = {'status_code': -1, 'id': resource_id, 'result': None}
        return ret_val

    def _aggregate_responses(self, done):
        result = []
        for item in done:
            result.append(item.result())
        return result

    def path_calls(self, URL, resource_ids):
        with ThreadPoolExecutor(self.number_of_threads) as pool:
            futures = [pool.submit(self._get_single_value, URL, resource_id) for resource_id in resource_ids]
            done, _ = wait(futures)
        return self._aggregate_responses(done)

    def query_param_calls(self, URL, query_param_name,  resource_ids):
        with ThreadPoolExecutor(self.number_of_threads) as pool:
            futures = [pool.submit(self._get_single_value, URL, resource_id, query_param_name) for resource_id in resource_ids]
            done, _ = wait(futures)
        return self._aggregate_responses(done)